#!/usr/bin/perl
#
# Genera el indice de icons y su extension asociada.
# No actua en los directorios donde existe el archivo .noindex 
# (no importa si tiene contenido o no)
#

use File::stat;
use POSIX qw(strftime);

my $dir = ".";
my $parent = "..";
my $cols = 4;  # Generamos una tabla con 4 columnas
my $mime   = "$ENV{HOME}/calcifer.org/icons/crux/mime-types";
my $imgdir = "/icons/crux";
my $themes = "/css/styles-site.css"; 
my $favicon = "/favicon.ico"; 

if ($ARGV[0]) {
	$dir = $ARGV[0];
}

if ($ARGV[1]) {
	$parent = $ARGV[1];
}

# Limpiamos la ruta
$dir =~ s|//||g;
$dir =~ s|/$||g;

$title = $dir;
$title =~ s|/|::|g;
$titleparent = $dir;
$titleparent =~ s|^(.*/)(.*)$|$1|g; # dirname

if ($titleparent !~ /(.*)\/(.*)/) {
	$titleparent = "..";
}

#
# Si existe .noindex es porque no se desea generar 
# el archivo HTML con el contenido del directorio
#
if (-f "$dir/.noindex") {
	exit;
}

opendir (DIR, $dir) || die "can't open $dir: $!";
my @dirs = grep {!/^.$/ && !/^..$/ 
			  && !/^\../ && !/^index.html$/
		  	  && !/^HEADER(\.html)?$/} readdir (DIR);
closedir (DIR);

@dirs = sort_dir ($dir, \@dirs);

my %r = load_mime_types ($mime);
my $filas = $#dirs / $cols;
my $fila = 0;

# Taman~o proporcional de cada columna en base al 90%,
# ya que la primera columna (del directorio padre) tiene
# un 10%.
my $percent = sprintf ("%d\%", 90 / $cols);

open (HTML, ">$dir/index.html");

my $iconparent = $r{"directory-dir"};


my $html_rows = 1;
my $html = get_html_dirs ();

my $header = qq{
	<html>
	<head>
	  <title>$title</title>
	  <link rel="stylesheet" href="$themes" type="text/css">
	  <link rel="icon" href="$favicon" type="image/x-icon">
	  <link rel="shortcut icon" href="$favicon" type="image/x-icon">
	</head>
	<body>

	<div class="bar-header">
		<div class="nav-links">
			<a href='/'>Inicio</a> 
			<a href='/weblog/'>bit&aacute;cora</a>
			<a href='/registro-fotografico/'>registro fotogr&aacute;fico</a>
			<a href='/chilelindo/'>chilelindo</a>
			<a href='/documentos/'>documentos</a>
			<a href='/archivos/'>archivos</a>
		</div>
	</div>

	<div id="banner"> $title </div>
	<hr noshade>
	<table align="center" width="100%" cellspacing="3" cellpadding="3">
	<tr>
		<td valign="top" align="center" bgcolor="#dedede" rowspan="$html_rows" width="20%">
			<a href="$parent">
			<img src="$imgdir/$iconparent" border="0"></a><br>$titleparent
		</td>
};

$footer = qq{
		</table>
		<script src="http://www.google-analytics.com/urchin.js" type="text/javascript">
		</script>
		<script type="text/javascript">
			_uacct = "UA-1374714-3";
			urchinTracker();
		</script>
	</body>
	</html>
	};

print HTML $header;
print HTML $html;
print HTML $footer;

close (HTML);
print "$dir/index.html\n";

sub get_html_dirs {
	#my $html = "	<tr>";
	my $html = "";
	
	foreach (@dirs) { 
		my $file = $_;
		my $image;
		my $size = "";

		$fila++;

		# Obtenemos informacion del archivo
		$detail = stat ("$dir/$file");
		$mtime = strftime "%e/%b/%Y, %H:%M", localtime $detail->mtime;

		if (-d "$dir/$file") {
			# es directorio
			$type = "directory";
			system ($0, "$dir/$file");
		} else {
			# obtenemos la extension del archivo
			@t = split (/\./, $file);
			$type = $t[$#t];
			$size  = sprintf ("%.1f KB", $detail->size / 1024);
			$mtime = "$size<br>$mtime";
		}

		if ($r{$type}) {
			$image = $r{$type}, "\n";
		} else {
			$image = $r{"text"};
		}

		$html .= qq{
			<td width="$percent" align="center">
				<span class="icon-item"><a href="$file">
				<img src="$imgdir/$image" alt="$type" border="0"><br>$file</a></span><br>
				<div class="icon-info">$mtime</div></a>
			</td>
		};

		if ((($fila % $cols) == 0) && ($fila < $#dirs + 1)) {
			$html .= "</tr><tr>";
			$html_rows++;
		}
	};

	while ($fila % $cols > 0) {
		$html .= "	<td width=\"$percent\" >&nbsp;</td>";
		$fila++;
	}

	# En caso de no existir archivos, rellenamos las columnas
	if ($fila == 0) {
		for ($i = 0; $i < $cols; $i++) {
			$html .= "	<td width=\"$percent\" >&nbsp;</td>";
		}
	}
	$html .= "	</tr>";

	return $html;
};

#
# Cargar las asociaciones entre extension del archivo
# y su correspondiente icono.
#
sub load_mime_types {
	my %result;

	open (MIME, @_[0]) || die "can't open $mime: $!";

	while (<MIME>) {
		chomp;
		split /:/;

		$result{$_[0]} = $_[1];
	}

	close (MIME);

	return %result;
}

# 
# Ordena un directorio, dejando en primer lugar
# los directorio y luego los archivos.  A su vez
# los ordena en forma ascendente dentro de su
# categoria.
#
sub sort_dir {
	my $basedir = $_[0];
	local (*alldirs)  = $_[1];

	my @dirs  = ();
	my @files = ();

	foreach (@alldirs) { 
		my $file = $_;

		if (-d "$basedir/$_") {
			push (@dirs, $_);
		} else {
			push (@files, $_);
		}
	}
	@dirs  = sort @dirs;
	@files = sort @files;

	push (@dirs, @files);

	return @dirs;
}
